import 'package:build_test/build_test.dart';
import 'package:source_gen/source_gen.dart' show LibraryReader;
import 'package:test/test.dart';
import 'package:researcher_builder/researcher_builder.dart';

import 'src/input/mock_assets.dart';
import 'src/mock_builders/mock_merging_builder.dart';

/// Tests if the generators [AddNumbersGenerator]
/// and [AddNamesGenerator] produce the expected output.
///
/// To run this program navigate to the top directory the package
/// [merging_builder] and use the command:
/// # pub run test -r expanded --test-randomize-ordering-seed=random
///
/// Note: The path to the input files is specified relative to the main
/// directory of [merging_builder].
Future<void> main() async {
  final libA = LibraryReader(await resolveSource(researcher_a_dot_dart,
      (resolver) => resolver.findLibraryByName('researcher_a')));

  final libB = LibraryReader(await resolveSource(researcher_b_dot_dart,
      (resolver) => resolver.findLibraryByName('researcher_b')));

  final numBuilder = MockMergingBuilder<num>(
    generator: AddNumbersGenerator(),
    libraries: [libA, libB],
    header: AddNumbersGenerator.header,
    footer: AddNumbersGenerator.footer,
    //formatOutput: (input) => (input),
  );

  final namesBuilder = MockMergingBuilder<List<String>>(
    generator: AddNamesGenerator(),
    libraries: [libA, libB],
    header: AddNamesGenerator.header,
    footer: AddNamesGenerator.footer,
    //formatOutput: (input) => (input),
  );

  final numMergedContent = await numBuilder.mergedContent;
  final numExpected =
      '// GENERATED CODE. DO NOT MODIFY. Generated by AddNumbersGenerator.\n'
      '\n'
      '/// Added numbers\n'
      'final num number0 = 19;\n'
      'final num number0 = 119;\n'
      'final num sum = 138;\n'
      '\n'
      '/// This is the footer.\n'
      '';

  /// This is the footer.
  final namesMergedContent = await namesBuilder.mergedContent;
  final namesExpected =
      '// GENERATED CODE. DO NOT MODIFY. Generated by AddNamesGenerator.\n'
      '\n'
      '/// Added names.\n'
      'final name0 = [\'Thomas\', \'Mayor\'];\n'
      'final name1 = [\'Philip\', \'Martens\'];\n'
      '\n'
      'final List<List<String>> names = [\n'
      '  [\'Thomas\', \'Mayor\'],\n'
      '  [\'Philip\', \'Martens\'],\n'
      '];\n'
      '\n'
      '/// This is the footer.\n'
      '';

  group('MergingGenerator:', () {
    test('Stream<List<String>>', () {
      expect(namesMergedContent, namesExpected);
    });
    test('Stream<num>', () {
      expect(numMergedContent, numExpected);
    });
  });
}
